from collections import namedtuple
from rtree import tree
from rtree.settings import Settings
from rtree.io_counter import Counter
from tqdm import trange, tqdm
from rtree.nodetypes import Element
from utils import create_from_dataset, query_from_dataset
import random
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib import style

style.use('ggplot')


def main():
    counter = Counter()
    settings = Settings()
    # configurations of runs
    Config = namedtuple('Config', 'TYPE FANOUT MIN_FILL MAX_OVERFLOW P_GRADE CACHE_MAXLEN')
    config_runs = [

        Config('linear', 50, 4, 1, 0.3, 0),
        Config('linear', 50, 4, 1, 0.3, 100),
        Config('linear', 50, 4, 1, 0.3, 500),
        Config('linear', 50, 4, 1, 0.3, 1000),
        Config('linear', 50, 4, 1, 0.3, 5000),
        Config('linear', 50, 4, 1, 0.3, 10000),

        Config('overflow', 50, 4, 3, 0.3, 0),
        Config('overflow', 50, 4, 3, 0.3, 100),
        Config('overflow', 50, 4, 3, 0.3, 500),
        Config('overflow', 50, 4, 3, 0.3, 1000),
        Config('overflow', 50, 4, 3, 0.3, 5000),
        Config('overflow', 50, 4, 3, 0.3, 10000),

        Config('star', 50, 4, 3, 0.3, 0),
        Config('star', 50, 4, 3, 0.3, 100),
        Config('star', 50, 4, 3, 0.3, 500),
        Config('star', 50, 4, 3, 0.3, 1000),
        Config('star', 50, 4, 3, 0.3, 5000),
        Config('star', 50, 4, 3, 0.3, 10000),

        Config('overflow-star', 50, 4, 3, 0.3, 0),
        Config('overflow-star', 50, 4, 3, 0.3, 100),
        Config('overflow-star', 50, 4, 3, 0.3, 500),
        Config('overflow-star', 50, 4, 3, 0.3, 1000),
        Config('overflow-star', 50, 4, 3, 0.3, 5000),
        Config('overflow-star', 50, 4, 3, 0.3, 10000),

    ]

    df = pd.DataFrame()
    for run in tqdm(config_runs):
        # Updating settings
        random.seed(3)

        settings.FANOUT = run.FANOUT
        settings.MIN_FILL = int(run.FANOUT * 0.4)  # all done with 40% fanout
        settings.MAX_OVERFLOW = run.MAX_OVERFLOW
        settings.P_GRADE = run.P_GRADE
        settings.CACHE_MAXLEN = run.CACHE_MAXLEN

        # Reset counter
        counter.reset()
        r_tree = tree.Tree(type=run.TYPE)

        # for i in trange(20000):
        create_from_dataset(r_tree, 'data/uni02.txt')

        df_run = pd.DataFrame(
            {
                'writes': counter.writes,
                'reads': counter.reads,
                'type': run.TYPE
            },
            index=[settings.CACHE_MAXLEN]
        )
        counter.reset()

        result = query_from_dataset(r_tree, 'query0/uni02.txt')
        print(result)
        df_run['q0'] = counter.reads/result.queries
        counter.reset()

        result = query_from_dataset(r_tree, 'query2/uni02.txt')
        print(result)
        df_run['q2'] = counter.reads/result.queries
        counter.reset()

        result = query_from_dataset(r_tree, 'query3/uni02.txt')
        print(result)
        df_run['q3'] = counter.reads/result.queries
        counter.reset()

        df = df.append(df_run)

    # plotting from the df
    plt.figure(0)
    df.loc[df['type'] == 'star']['writes'].plot(label='Star', style='s-')
    df.loc[df['type'] == 'overflow']['writes'].plot(label='Overflow-linear3', style='s-')
    df.loc[df['type'] == 'linear']['writes'].plot(label='Linear', style='s-')
    df.loc[df['type'] == 'overflow-star']['writes'].plot(label='Overflow-star3', style='s-')
    plt.legend()
    plt.xticks([10, 50, 100, 500, 1000, 5000, 10000])
    plt.savefig('graphs/uni02_change_cachesize_writes')

    plt.figure(1)
    df.loc[df['type'] == 'star']['reads'].plot(label='Star', style='s-')
    df.loc[df['type'] == 'overflow']['reads'].plot(label='Overflow-linear3', style='s-')
    df.loc[df['type'] == 'linear']['reads'].plot(label='Linear', style='s-')
    df.loc[df['type'] == 'overflow-star']['reads'].plot(label='Overflow-star3', style='s-')
    plt.legend()
    plt.xticks([10, 50, 100, 500, 1000, 5000, 10000])
    plt.savefig('graphs/uni02_change_cachesize_reads')

    plt.figure(2)
    df.loc[df['type'] == 'star']['q0'].plot(label='Star', style='s-')
    df.loc[df['type'] == 'overflow']['q0'].plot(label='Overflow-linear3', style='s-')
    df.loc[df['type'] == 'linear']['q0'].plot(label='Linear', style='s-')
    df.loc[df['type'] == 'overflow-star']['q0'].plot(label='Overflow-star3', style='s-')
    plt.legend()
    plt.xticks([10, 50, 100, 500, 1000, 5000, 10000])
    plt.savefig('graphs/uni02_change_cachesize_q0')

    plt.figure(3)
    df.loc[df['type'] == 'star']['q2'].plot(label='Star', style='s-')
    df.loc[df['type'] == 'overflow']['q2'].plot(label='Overflow-linear3', style='s-')
    df.loc[df['type'] == 'linear']['q2'].plot(label='Linear', style='s-')
    df.loc[df['type'] == 'overflow-star']['q2'].plot(label='Overflow-star3', style='s-')
    plt.legend()
    plt.xticks([10, 50, 100, 500, 1000, 5000, 10000])
    plt.savefig('graphs/uni02_change_cachesize_q2')

    plt.figure(4)
    df.loc[df['type'] == 'star']['q3'].plot(label='Star', style='s-')
    df.loc[df['type'] == 'overflow']['q3'].plot(label='Overflow-linear3', style='s-')
    df.loc[df['type'] == 'linear']['q3'].plot(label='Linear', style='s-')
    df.loc[df['type'] == 'overflow-star']['q3'].plot(label='Overflow-star3', style='s-')
    plt.legend()
    plt.xticks([10, 50, 100, 500, 1000, 5000, 10000])
    plt.savefig('graphs/uni02_change_cachesize_q3')

    df.to_pickle('graphs/uni02_' + str(__file__[:-1]))  # saving as pickle as filename.p
    print(df)


def create_random_element(text='random generated node'):
    x = random.randint(0, 1000)
    y = random.randint(0, 1000)
    return Element((x, y), text)


if __name__ == '__main__':
    main()
