from collections import namedtuple
from rtree import tree
from rtree.settings import Settings
from rtree.io_counter import Counter
from tqdm import trange, tqdm
from rtree.nodetypes import Element
from utils import create_from_dataset, query_from_dataset
import random
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib import style

style.use('ggplot')


def main():
    counter = Counter()
    settings = Settings()
    # configurations of runs
    Config = namedtuple('Config', 'TYPE FANOUT MIN_FILL MAX_OVERFLOW P_GRADE CACHE_MAXLEN')
    config_runs = [

        Config('linear', 10, 4, 1, 0.3, 1000),
        Config('linear', 30, 4, 1, 0.3, 1000),
        Config('linear', 50, 4, 1, 0.3, 1000),
        Config('linear', 70, 4, 1, 0.3, 1000),
        Config('linear', 100, 4, 1, 0.3, 1000),

        Config('overflow', 10, 4, 2, 0.3, 1000),
        Config('overflow', 30, 4, 2, 0.3, 1000),
        Config('overflow', 50, 4, 2, 0.3, 1000),
        Config('overflow', 70, 4, 2, 0.3, 1000),
        Config('overflow', 100, 4, 2, 0.3, 1000),

        Config('overflow', 10, 4, 3, 0.3, 1000),
        Config('overflow', 30, 4, 3, 0.3, 1000),
        Config('overflow', 50, 4, 3, 0.3, 1000),
        Config('overflow', 70, 4, 3, 0.3, 1000),
        Config('overflow', 100, 4, 3, 0.3, 1000),

        Config('overflow', 10, 4, 4, 0.3, 1000),
        Config('overflow', 30, 4, 4, 0.3, 1000),
        Config('overflow', 50, 4, 4, 0.3, 1000),
        Config('overflow', 70, 4, 4, 0.3, 1000),
        Config('overflow', 100, 4, 4, 0.3, 1000),

        Config('overflow', 10, 4, 5, 0.3, 1000),
        Config('overflow', 30, 4, 5, 0.3, 1000),
        Config('overflow', 50, 4, 5, 0.3, 1000),
        Config('overflow', 70, 4, 5, 0.3, 1000),
        Config('overflow', 100, 4, 5, 0.3, 1000),

        Config('overflow', 10, 4, 6, 0.3, 1000),
        Config('overflow', 30, 4, 6, 0.3, 1000),
        Config('overflow', 50, 4, 6, 0.3, 1000),
        Config('overflow', 70, 4, 6, 0.3, 1000),
        Config('overflow', 100, 4, 6, 0.3, 1000),
    ]

    df = pd.DataFrame()
    for run in tqdm(config_runs):
        # Updating settings
        random.seed(3)

        settings.FANOUT = run.FANOUT
        settings.MIN_FILL = int(run.FANOUT * 0.4)  # all done with 40% fanout
        settings.MAX_OVERFLOW = run.MAX_OVERFLOW
        settings.P_GRADE = run.P_GRADE
        settings.CACHE_MAXLEN = run.CACHE_MAXLEN

        # Reset counter
        counter.reset()
        r_tree = tree.Tree(type=run.TYPE)

        # for i in trange(20000):
        create_from_dataset(r_tree, 'data/uni02.txt')

        df_run = pd.DataFrame(
            {
                'blocks': settings.MAX_OVERFLOW,
                'writes': counter.writes,
                'reads': counter.reads
            },
            index=[settings.FANOUT]
        )
        counter.reset()

        result = query_from_dataset(r_tree, 'query0/uni02.txt')
        print(result)
        df_run['q0'] = counter.reads/result.queries
        counter.reset()

        result = query_from_dataset(r_tree, 'query2/uni02.txt')
        print(result)
        df_run['q2'] = counter.reads/result.queries
        counter.reset()

        result = query_from_dataset(r_tree, 'query3/uni02.txt')
        print(result)
        df_run['q3'] = counter.reads/result.queries
        counter.reset()

        df = df.append(df_run)

    # plotting from the df
    """
    plt.figure(0)
    df.loc[df['blocks'] == 1]['writes'].plot(label='Linear', style='s-')
    df.loc[df['blocks'] == 2]['writes'].plot(label='OF2', style='s-')
    df.loc[df['blocks'] == 3]['writes'].plot(label='OF3', style='s-')
    df.loc[df['blocks'] == 4]['writes'].plot(label='OF4', style='s-')
    df.loc[df['blocks'] == 5]['writes'].plot(label='OF5', style='s-')
    df.loc[df['blocks'] == 6]['writes'].plot(label='OF6', style='s-')
    plt.legend()
    plt.xticks([10, 30, 50, 70, 100])
    plt.savefig('graphs/uni02_change_overflowblocks_linear_writes')

    plt.figure(1)
    df.loc[df['blocks'] == 1]['reads'].plot(label='Linear', style='s-')
    df.loc[df['blocks'] == 2]['reads'].plot(label='OF2', style='s-')
    df.loc[df['blocks'] == 3]['reads'].plot(label='OF3', style='s-')
    df.loc[df['blocks'] == 4]['reads'].plot(label='OF4', style='s-')
    df.loc[df['blocks'] == 5]['reads'].plot(label='OF5', style='s-')
    df.loc[df['blocks'] == 6]['reads'].plot(label='OF6', style='s-')
    plt.legend()
    plt.xticks([10, 30, 50, 70, 100])
    plt.savefig('graphs/uni02_change_overflowblocks_linear_reads')

    plt.figure(2)
    df.loc[df['blocks'] == 1]['q0'].plot(label='Linear', style='s-')
    df.loc[df['blocks'] == 2]['q0'].plot(label='OF2', style='s-')
    df.loc[df['blocks'] == 3]['q0'].plot(label='OF3', style='s-')
    df.loc[df['blocks'] == 4]['q0'].plot(label='OF4', style='s-')
    df.loc[df['blocks'] == 5]['q0'].plot(label='OF5', style='s-')
    df.loc[df['blocks'] == 6]['q0'].plot(label='OF6', style='s-')
    plt.legend()
    plt.xticks([10, 30, 50, 70, 100])
    plt.savefig('graphs/uni02_change_overflowblocks_linear_q0')

    plt.figure(3)
    df.loc[df['blocks'] == 1]['q2'].plot(label='Linear', style='s-')
    df.loc[df['blocks'] == 2]['q2'].plot(label='OF2', style='s-')
    df.loc[df['blocks'] == 3]['q2'].plot(label='OF3', style='s-')
    df.loc[df['blocks'] == 4]['q2'].plot(label='OF4', style='s-')
    df.loc[df['blocks'] == 5]['q2'].plot(label='OF5', style='s-')
    df.loc[df['blocks'] == 6]['q2'].plot(label='OF6', style='s-')
    plt.legend()
    plt.xticks([10, 30, 50, 70, 100])
    plt.savefig('graphs/uni02_change_overflowblocks_linear_q2')

    plt.figure(4)
    df.loc[df['blocks'] == 1]['q3'].plot(label='Linear', style='s-')
    df.loc[df['blocks'] == 2]['q3'].plot(label='OF2', style='s-')
    df.loc[df['blocks'] == 3]['q3'].plot(label='OF3', style='s-')
    df.loc[df['blocks'] == 4]['q3'].plot(label='OF4', style='s-')
    df.loc[df['blocks'] == 5]['q3'].plot(label='OF5', style='s-')
    df.loc[df['blocks'] == 6]['q3'].plot(label='OF6', style='s-')
    plt.legend()
    plt.xticks([10, 30, 50, 70, 100])
    plt.savefig('graphs/uni02_change_overflowblocks_linear_q3')
    """
    df.to_pickle('graphs/uni02_1000_' + str(__file__[:-1]))  # saving as pickle as filename.p
    print(df)


def create_random_element(text='random generated node'):
    x = random.randint(0, 1000)
    y = random.randint(0, 1000)
    return Element((x, y), text)


if __name__ == '__main__':
    main()
