from collections import namedtuple
from rtree import tree
from rtree.settings import Settings
from rtree.io_counter import Counter
from tqdm import trange, tqdm
from rtree.nodetypes import Element
from utils import create_from_dataset, query_from_dataset
import random
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib import style

style.use('ggplot')


def main():
    counter = Counter()
    settings = Settings()
    # configurations of runs
    Config = namedtuple('Config', 'TYPE FANOUT MIN_FILL MAX_OVERFLOW P_GRADE CACHE_MAXLEN')
    config_runs = [
        Config('overflow-star', 10, 4, 5, 0.3, 1000),
        Config('overflow-star', 30, 4, 5, 0.3, 1000),
        Config('overflow-star', 50, 4, 5, 0.3, 1000),
        Config('overflow-star', 70, 4, 5, 0.3, 1000),
        Config('overflow-star', 100, 4, 5, 0.3, 1000),

        Config('overflow-star', 10, 4, 6, 0.3, 1000),
        Config('overflow-star', 30, 4, 6, 0.3, 1000),
        Config('overflow-star', 50, 4, 6, 0.3, 1000),
        Config('overflow-star', 70, 4, 6, 0.3, 1000),
        Config('overflow-star', 100, 4, 6, 0.3, 1000),
    ]

    df = pd.DataFrame()
    for run in tqdm(config_runs):
        # Updating settings
        random.seed(3)

        settings.FANOUT = run.FANOUT
        settings.MIN_FILL = int(run.FANOUT * 0.4)  # all done with 40% fanout
        settings.MAX_OVERFLOW = run.MAX_OVERFLOW
        settings.P_GRADE = run.P_GRADE
        settings.CACHE_MAXLEN = run.CACHE_MAXLEN

        # Reset counter
        counter.reset()
        r_tree = tree.Tree(type=run.TYPE)

        # for i in trange(20000):
        create_from_dataset(r_tree, 'data/uni02.txt')

        df_run = pd.DataFrame(
            {
                'blocks': settings.MAX_OVERFLOW,
                'writes': counter.writes,
                'reads': counter.reads
            },
            index=[settings.FANOUT]
        )
        counter.reset()

        result = query_from_dataset(r_tree, 'query0/uni02.txt')
        print(result)
        df_run['q0'] = counter.reads/result.queries
        counter.reset()

        result = query_from_dataset(r_tree, 'query2/uni02.txt')
        print(result)
        df_run['q2'] = counter.reads/result.queries
        counter.reset()

        result = query_from_dataset(r_tree, 'query3/uni02.txt')
        print(result)
        df_run['q3'] = counter.reads/result.queries
        counter.reset()

        df = df.append(df_run)

    df.to_pickle('graphs/uni02_1000_3_' + str(__file__[:-1]))  # saving as pickle as filename.p
    print(df)


def create_random_element(text='random generated node'):
    x = random.randint(0, 1000)
    y = random.randint(0, 1000)
    return Element((x, y), text)


if __name__ == '__main__':
    main()
