from collections import namedtuple
from rtree import tree
from rtree.settings import Settings
from rtree.io_counter import Counter
from tqdm import trange, tqdm
from rtree.nodetypes import Element
from utils import create_from_dataset, query_from_dataset
import random
import matplotlib

def main():
    counter = Counter()
    settings = Settings()
    # configurations of runs
    Config = namedtuple('Config', 'TYPE FANOUT MIN_FILL MAX_OVERFLOW P_GRADE CACHE_MAXLEN')
    config_runs = [
        Config('overflow', 10, 4, 4, 0.3, 0),
        Config('linear', 10, 4, 4, 0.3, 0),
        Config('star', 10, 4, 4, 0.3, 0),
        Config('overflow-star', 10, 4, 4, 0.3, 0),

        Config('overflow', 30, 4, 4, 0.3, 0),
        Config('linear', 30, 4, 4, 0.3, 0),
        Config('star', 30, 4, 4, 0.3, 0),
        Config('overflow-star', 30, 4, 4, 0.3, 0),

        Config('overflow', 50, 4, 4, 0.3, 0),
        Config('linear', 50, 4, 4, 0.3, 0),
        Config('star', 50, 4, 4, 0.3, 0),
        Config('overflow-star', 50, 4, 4, 0.3, 0),

        Config('overflow', 70, 4, 4, 0.3, 0),
        Config('linear', 70, 4, 4, 0.3, 0),
        Config('star', 70, 4, 4, 0.3, 0),
        Config('overflow-star', 70, 4, 4, 0.3, 0),

        Config('overflow', 100, 4, 4, 0.3, 0),
        Config('linear', 100, 4, 4, 0.3, 0),
        Config('star', 100, 4, 4, 0.3, 0),
        Config('overflow-star', 100, 4, 4, 0.3, 0),

        # Config('overflow', 100, 40, 4, 0.3, 10),
        # Config('linear', 100, 40, 4, 0.3, 10),
        # Config('star', 100, 40, 4, 0.3, 10),
    ]

    for run in config_runs:
        # Updating settings
        random.seed(3)

        settings.FANOUT = run.FANOUT
        settings.MIN_FILL = int(run.FANOUT*0.4)
        settings.MAX_OVERFLOW = run.MAX_OVERFLOW
        settings.P_GRADE = run.P_GRADE
        settings.CACHE_MAXLEN = run.CACHE_MAXLEN

        # Reset counter
        counter.reset()

        print(run)

        r_tree = tree.Tree(type=run.TYPE)

        for i in trange(20000):
            # print("mainloop", i)
            e = create_random_element()
            r_tree.insert_element(e)

        levels = r_tree.levels_count()
        print()
        for l in sorted(levels):
            print(l, levels[l])

        print('reads: {}, writes: {}'.format(counter.reads, counter.writes))
        print('\n' * 3)


def create_random_element(text='random generated node'):
    x = random.randint(0, 1000)
    y = random.randint(0, 1000)
    return Element((x, y), text)


if __name__ == '__main__':
    main()
