from tqdm import trange, tqdm
from rtree.nodetypes import Element, BF_Node, BF_OverflowNode
from rtree.tree import Tree
import random
from rtree.io_counter import Counter
from utils import get_dataset
from rtree.mbr import MBR


def main():
    counter = Counter()
    r_tree = Tree(type='star')
    # r_tree = Tree(type='linear')
    # r_tree = Tree(type='star')
    # print(r_tree.root)

    for i in trange(2000):
        # print("mainloop", i)
        e = create_random_element()
        r_tree.insert_element(e)
        # print_tree(r_tree.root)



    """
    # Inserting data
    pbar = tqdm(total=1000000, desc='Inserting into tree')
    for x_range, y_range in get_dataset('distributions/data/bit02.txt'):
        e = Element((x_range[0], y_range[1]), data="x")
        r_tree.insert_element(e)
        pbar.update()
    pbar.close()

    # Querying data
    total_answers = 0
    error = 0
    pbar = tqdm(total=100000, desc='Running query0 (one result pr query)')
    for x_range, y_range in get_dataset('distributions/query0/bit02.txt'):
        result = r_tree.search(MBR(x_range[0], x_range[1], y_range[0], y_range[1]))
        total_answers += len(result)
        pbar.update()
        if len(result) != 1:
            error += 1
    pbar.close()
    print('Total answers', total_answers, "| average:", total_answers/100000)
    print('total_errors', error)
    """

    # r_tree.delete(e)
    print('x'*40)
    # print_tree(r_tree.root)

    levels = r_tree.levels_count()
    for l in sorted(levels):
        print(l, levels[l])
    plot_tree(r_tree.root)
    print('reads: {}, writes: {}'.format(counter.reads, counter.writes))


def create_random_element(text='random generated node'):
    x = random.randint(0, 1000)
    y = random.randint(0, 1000)
    return Element((x, y), text)


def print_tree(node, indention_level=2):
    # Do a DFS reccursivly
    if indention_level == 2:
        print(node, repr(node))
    indent = ' ' * indention_level
    if node.is_leaf():
        for child in node.children:
            print('{0}{1}'.format(indent, child), repr(child))
    else:
        for child in node.children:
            print('{0}{1}'.format(indent, child), repr(child), repr(child.parent))
            print_tree(child, indention_level + 2)


def plot_tree(root_node):
    print("potting")
    import matplotlib.pyplot as plt
    import matplotlib.patches as patches
    from collections import deque

    bfs_queue = deque()
    bfs_queue.appendleft(root_node)
    i = 0
    # markers = ('o', 'v', '^', '<', '>', '8', 's', 'p', '*', 'h', 'H', 'D', 'd', 'P', 'X')
    fig1 = plt.figure()
    ax1 = fig1.add_subplot(111, aspect='equal')
    while bfs_queue:
        current = bfs_queue.pop()
        if current.is_leaf():
            ax1.add_patch(
                patches.Rectangle(
                    (current.mbr.x_min, current.mbr.y_min),  # (x,y)
                    current.mbr.x_max - current.mbr.x_min,  # width
                    current.mbr.y_max - current.mbr.y_min,  # height
                    fill=True,
                    alpha=0.1,
                    color='red',
                    linewidth=0,
                )
            )
            for leaf_element in current.children:
                ax1.plot(*leaf_element.pos, 'ko', ms=0.7, alpha=1, fillstyle='full')
                # ax1.plot(*leaf_element.pos, 'g'+markers[i%len(markers)])
            i += 1
        else:
            """
            ax1.add_patch(
                patches.Rectangle(
                    (current.mbr.x_min, current.mbr.y_min),  # (x,y)
                    current.mbr.x_max - current.mbr.x_min,  # width
                    current.mbr.y_max - current.mbr.y_min,  # height
                    fill=False,
                    linewidth=1,
                )
            )
            """
            bfs_queue.extendleft(current.children)

    plt.axis('off')
    plt.subplots_adjust(left=0.0, right=1, top=1, bottom=0)
    fig1.savefig('rect1.png', dpi=300, bbox_inches='tight')


if __name__ == '__main__':
    random.seed(3)
    main()
