from math import sqrt
from collections import deque

from .settings import Settings
from .mbr import MBR
from .io_counter import Counter

from .nodetypes import Element, Node, BF_Node, BF_OverflowNode, BF_OverflowNode_star, BF_Node_star

counter = Counter()
settings = Settings()


def range_search(Q, RN):
    """
    Find all elements in the tree that are within Q, Q is an mbr, RN is initially root node
    """
    answer = set()
    if RN.is_leaf():

        if type(RN) is BF_OverflowNode or type(RN) is BF_OverflowNode_star:
            for block in RN.overflow_blocks:
                if block: # If it's not empty..
                    counter.read(block)
                    for element in block:
                        if Q.overlaps(element.mbr):
                            answer.add(element)

        else:
            counter.read(RN)
            for element in RN.children:
                if Q.overlaps(element.mbr):
                    answer.add(element)

    else:
        counter.read(RN)
        for child in RN.children:
            if Q.overlaps(child.mbr):
                answer = answer.union(range_search(Q, child))
    return answer


def insert(E, RN):
    """
    Insert element E into the tree. Finding the appropriate leafnode for insertion
    """

    # TODO: remame to N, to be more consistent?
    suited_node = choose_subtree(E, RN)
    suited_node.insert_element(E)


def insert_star(E, RN, called_levels=None):
    if called_levels is None:
        called_levels = ()

    if type(E) is Element:
        N = choose_subtree_star(E, RN)
        N.insert_element(E, called_levels)
    elif isinstance(E, Node):
        N = choose_subtree_star(E, RN)
        N.insert_element(E, called_levels)
    else:
        raise ValueError("oooh fuck")
    # TODO tackle insertions of branches


def insert_bf(E, RN):
    # own method basicly to take into account that the objects are different type. Workings are same as regular bf
    suited_node = choose_subtree_bf(E, RN)
    suited_node.insert_element(E)


def choose_subtree_bf(E, RN):
    # same as normal choose_subtree, except it checks for different

    N = RN
    # a read for root node, or is it always in memory?
    if type(E) is Element:
        while N.is_leaf() is False:
            # sort on area to circumvent E, then area of mbr.
            N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
            counter.read(N)
    elif type(E) is BF_Node:
        while N.get_height() != E.get_height() + 1:
            # sort on area to circumvent E, then area of mbr.
            N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
            counter.read(N)
    else:
        print("please dont")
    return N


def insert_star_bf(E, RN):
    N = choose_subtree_star(E, RN)
    N.insert_element(E)


def choose_subtree_star_bf(E, RN):
    # The choose subtree for the R* tree.
    N = RN

    if type(E) is Element:

        while N.is_leaf() is False:
            if all([c.is_leaf() for c in N.children]):
                N = sorted(N.children, key=lambda x: (overlap_enlargement(x, N.children, E), x.mbr.area()))[0]
            else:
                N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
            counter.read(N)
    elif type(E) is BF_Node_star:
        while N.get_height() != E.get_height() + 1:
            # sort on area to circumvent E, then area of mbr.
            N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
            counter.read(N)

    return N


def choose_subtree(E, RN):
    # Own function to allow it to be plugable. Guttmanns tree.
    N = RN
    # a read for root node, or is it always in memory?
    while N.is_leaf() is False:
        # sort on area to circumvent E, then area of mbr.
        N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
        counter.read(N)

    return N


def choose_subtree_star(E, RN):
    # The choose subtree for the R* tree.
    # TODO: bedre beskrivelse
    N = RN
    if type(E) is Element:
        while N.is_leaf() is False :
            if all([c.is_leaf() for c in N.children]):
                N = sorted(N.children, key=lambda x: (overlap_enlargement(x, N.children, E), x.mbr.area()))[0]
            else:
                N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
            counter.read(N)

    # Info: N-height 1 never happens because it becommes two after a split. 1 never compared to 1
    # In order to insert branches, happens when inserting pointers higher than leaf level.
    elif isinstance(E, Node):
        while N.get_height() != E.get_height() + 1:
            if all([c.is_leaf() for c in N.children]):
                N = sorted(N.children, key=lambda x: (overlap_enlargement(x, N.children, E), x.mbr.area()))[0]
            else:
                N = sorted(N.children, key=lambda x: (x.mbr.area_to_circumvent_mbr(E.mbr), x.mbr.area()))[0]
            counter.read(N)
    else:
        raise Exception('E is neither Element nor Node')
    return N


def overlap_enlargement(N, siblings, E):
    # siblings includes itself
    if N.mbr.area_to_circumvent_mbr(E.mbr) == 0:
        return 0

    overlap_before = 0
    for c in siblings:
        if c is N:
            continue
        overlap_before += N.mbr.overlap(c.mbr)

    x_values = [N.mbr.x_min, N.mbr.x_max, E.mbr.x_min, E.mbr.x_max]
    y_values = [N.mbr.y_min, N.mbr.y_max, E.mbr.y_min, E.mbr.y_max]

    new_mbr = MBR(min(x_values), max(x_values), min(y_values), max(y_values))
    overlap_after = 0

    for c in siblings:
        if c is N:
            continue
        overlap_after += new_mbr.overlap(c.mbr)

    return overlap_after - overlap_before


def delete(E, RN):
    """
    Delete element E from the tree
    """
    # Done with queue insted of recursivly to easily break of when correct leaf is found

    dfs_queue = deque()
    dfs_queue.append(RN)

    while dfs_queue:
        current = dfs_queue.pop()
        if current.is_leaf() and E in current.children:
            current.remove_element(E)
            condense_tree(current)
            break
        elif current.is_leaf() is False:
            for child in current.children:
                if child.mbr.overlaps(E.mbr):
                    dfs_queue.append(child)
    else:
        print("Did not manage to delete element")

    if len(RN.children) == 1 and RN.is_leaf() is False:
        RN.children = RN.children[0].children
        for child in RN.children:
            child.parent = RN


def condense_tree(L):
    current = L
    N = set()
    while not current.is_root():
        parent = current.parent
        if len(current.children) < (settings.FANOUT/2):
            parent.remove_element(current)
            N.add(current)
        current = parent

    # Reinsert all entries
    for n in N:
        if type(n) is Element:
            insert(n, current)
        elif type(n) is Node:
            for e in get_entries_from_branch(n):
                insert(e, current)
        else:
            raise("Unknown type to insert:", n, "is ", repr(n))


def get_entries_from_branch(L):
    entries = set()
    if L.is_leaf():
        return set(L.children)

    for child in L.children:
        entries = entries.union(get_entries_from_branch(child))

    return entries


def euclidian_distance(a, b):  # TODO: move to helper?
    """Returns the distance between two 2D points"""
    return sqrt((a[0] - b[0]) ** 2 + (a[1] - b[1]) ** 2)
