from collections import deque
from rtree.settings import Settings
settings = Settings()


class Counter(object):
    """
    Borg design pattern instead of singleton.
    Cache to reduce reads. Implemented as a simple LRU cache
    """
    __state = {'reads': 0, 'writes': 0, 'cache': deque(maxlen=settings.CACHE_MAXLEN)}

    def __init__(self):
        self.__dict__ = self.__state

    def read(self, node):
        """
        Method that simulates a read in a system with a cache. If node doesn't exist in cache, then read count is
        incremented and the node is put in cache.
        :param node:
        :return: True if read is incremented, False if it was in cache
        """

        if node in self.cache:
            self.cache.remove(node)
            self.cache.append(node)
            return False

        elif node not in self.cache:
            self.reads += 1
            self.cache.append(node)
            return True

    def write(self, node):
        """
        Simulates a write. Increments write count
        A write requires a read first.  # TODO: This might not be accurate when creating new nodes?
        :return:
        """
        self.read(node)
        self.writes += 1

    def reset(self):
        self.reads = 0
        self.writes = 0
        self.cache = deque(maxlen=settings.CACHE_MAXLEN)
