import pytest
from .tree import Tree
from .nodetypes import Node, Element, StarNode
from .mbr import MBR

def test_initial_tree_has_node():
    tree = Tree()

    assert type(tree.root) is Node


def test_initial_tree_doesnt_have_mbr():
    tree = Tree()

    assert tree.root.mbr is None


def test_initial_tree_is_root_and_leaf():
    tree = Tree()

    assert tree.root.is_root()
    assert tree.root.is_leaf()


def test_can_create_tree_with_StarNode():
    tree = Tree(type='star')

    assert type(tree.root) is StarNode


def test_tree_saves_elements():
    tree = Tree()
    element = Element((0, 1), 'new element')
    tree.insert_element(element)

    assert element in tree.root.children


def test_tree_updates_mbr():
    tree = Tree()
    element = Element((0, 1), 'new element')
    tree.insert_element(element)

    assert type(tree.root.mbr) is MBR


def test_can_retrieve_inserted_element():
    tree = Tree()
    element = Element((0, 1), 'new element')
    tree.insert_element(element)
    answerset = tree.search(MBR(0, 10, 0, 10))

    assert len(answerset) == 1


def test_can_delete_inserted_element():
    tree = Tree()
    element = Element((0, 1), 'new element')

    tree.insert_element(element)
    answerset = tree.search(MBR(0, 10, 0, 10))
    assert len(answerset) == 1

    tree.delete(element)
    answerset = tree.search(MBR(0, 10, 0, 10))
    assert len(answerset) == 0


def test_point_query_retrieves_point_if_exact_mbr_query():
    tree = Tree()

    element = Element((0, 1), 'new element')
    tree.insert_element(element)
    answerset = tree.search(MBR(0, 0, 1, 1))

    assert len(answerset) == 1
    assert element in answerset


def test_decimal_query_retrieves_point_if_exact_mrb_query():
    tree = Tree()

    x = float('2.661137581896043e-01')
    y = float('9.410341768245845e-03')
    element = Element((x, y), 'new element')
    tree.insert_element(element)
    answerset = tree.search(MBR(x, x, y, y))

    assert len(answerset) == 1
    assert element in answerset
