from .nodetypes import Node, StarNode, BF_Node, BF_OverflowNode, BF_OverflowNode_star, BF_Node_star
from .algorithms import range_search, insert, delete, insert_star, insert_star_bf, insert_bf
from .io_counter import Counter

counter = Counter()


class Tree():
    def __init__(self, type='linear'):
        self.type = type
        if self.type == 'linear':
            self.root = Node()
        elif self.type == 'star':
            self.root = StarNode()
        elif self.type == 'overflow':
            self.root = BF_Node()
        elif self.type == 'overflow-star':
            self.root = BF_Node_star()
        else:
            raise Exception('Please use a valid tree type')
        counter.writes += 1

    def search(self, Q):
        return range_search(Q, self.root)

    def insert_element(self, E):
        if self.type == 'linear':
            insert(E, self.root)
        elif self.type == 'star':
            insert_star(E, self.root)
        elif self.type == 'overflow':
            insert_bf(E, self.root)
        elif self.type == 'overflow-star':
            insert_star_bf(E, self.root)

    def delete(self, E):
        delete(E, RN=self.root)

    def levels_count(self):
        from collections import deque, defaultdict
        levels = defaultdict(int)
        dfs_queue = deque()
        dfs_queue.append((self.root, 0))
        while dfs_queue:
            current, level = dfs_queue.pop()
            levels[level] += 1
            if current.is_leaf() and (type(current) is BF_OverflowNode or type(current) is BF_OverflowNode_star):
                levels[level + 1] += current.element_count()
            elif current.is_leaf():
                levels[level + 1] += len(current.children)
            else:
                for c in current.children:
                    dfs_queue.append((c, level + 1))

        return levels
