from collections import namedtuple
from tqdm import trange, tqdm
from rtree.nodetypes import Element
from rtree.mbr import MBR


def get_dataset(path):
    with open(path) as f:
        # skip the first three lines
        f.readline()
        f.readline()
        f.readline()
        while True:
            x_line = f.readline()
            if x_line == '':
                break
            x_line = x_line.strip()
            try:
                xmin, xmax = (float(val) for val in x_line.split())
            except Exception as e:
                print("Error when reading line:", repr(x_line))
                raise e
            y_line = f.readline()
            ymin, ymax = [float(val) for val in y_line.split()]
            _ = f.readline()
            yield (xmin, xmax), (ymin, ymax)


def create_from_dataset(tree, dataset, show_progress=False):
    # Inserting data
    if show_progress:
        pbar = tqdm(total=1000000, desc='Inserting into tree')

    for x_range, y_range in get_dataset('distributions/'+dataset):
        e = Element((x_range[0], y_range[1]), data="x")
        tree.insert_element(e)
        if show_progress:
            pbar.update()

    if show_progress:
        pbar.close()


def query_from_dataset(tree, dataset, show_progress=False):
    total_answers = 0
    total_queries = 0
    if show_progress:
        pbar = tqdm(desc='Running ' + dataset)

    for x_range, y_range in get_dataset('distributions/'+dataset):
        result = tree.search(MBR(x_range[0], x_range[1], y_range[0], y_range[1]))
        total_answers += len(result)
        total_queries += 1

        if show_progress:
            pbar.update()

    if show_progress:
        pbar.close()

    Result = namedtuple('Result', 'answers queries avg')
    avg = total_answers/total_queries
    res = Result(total_answers, total_queries, avg)
    return res
